/***************************************************************************
 *   Copyright (C) 2007-2009 by Helge Kortendieck  alan@kasmithee.de       *
 *   aboutHelp.cpp  -  show about or help dialog                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#include "abouthelp.h"

aboutHelp::aboutHelp( QDialog* parent, Qt::WindowFlags fl )
        : QDialog( parent, fl )
{
  uih.setupUi( &wHelp );
  connect( uih.pB_Close, SIGNAL( clicked() ), this, SLOT( closeDlg() ) );
}

aboutHelp::~aboutHelp()
{
}

void aboutHelp::showType( int iD )              // show dialog, 1=about, 2=help
{
  iTyp = iD;
  if( iTyp == 1 ) {
    showAbout();
  } else if( iTyp == 2 ) {
    showHelp();
  } else if( iTyp == 3 ) {
    showErr();
  }
}

void aboutHelp::showAbout()                 // show about dialog, type 1
{
  iWidth = 525;
  iHeight = 180;
  sTitle = tr( "about qvtwfc" );
  showDlg();
}

void aboutHelp::showHelp()                  // show help dialog, type 2
{
  sTitle = tr( "qvtwfc - help" );
  showDlg();
}

void aboutHelp::showErr()                   // show error dialog, type 3
{
  iWidth = 550;
  iHeight = 750;
//  sTitle = tr( "qvtwfc - error" );            // already set from parent (qvtwfc.cpp/h)
  showDlg();
}

void aboutHelp::showDlg()                   // show help dialog
{
  if( wHelp.isVisible() == true ) {
    wHelp.hide();
  }
  setText();
  if( iTyp == 1 or iTyp == 2 ) {
    uih.tb_Help->setHtml( sText );
  } else {
    uih.tb_Help->setPlainText( sText );
  }
  wHelp.setWindowTitle( sTitle );
  if( iTyp == 2 ) {
    wHelp.showFullScreen();
  } else {
    wHelp.resize( iWidth, iHeight );
  }
  wHelp.show();
}

void aboutHelp::closeDlg()
{
  wHelp.hide();
  emit sigAhClosed();
}

void aboutHelp::setText()           // set text to show: 1=About (html), 2=Help (html), 3=Error (txt)
{
  switch ( iTyp ) {
    case 1:                                     // about text (html)
      sText = tr( "<html><body>"
                  "qvtwfc - Qt4 vocabulary training with flash cards.<br>"
                  "A simple flash card vocabulary trainer.<p>"
                  "Version 1.0.0.1, (C) 2007-2009 by Helge Kortendieck alan@kasmithee.de<br>"
                  "This program is released under GPL2 or later."
                  "</body></html>" );
      break;
    case 2:                                     // help text (html)
      sText = tr( "<html><body>"
                  "<a name=""Top""><b>Table of content</b></a><br>"
                  "<a href=""#Intro"">Introduction</a><br>"
                  "<a href=""#Start"">Starting</a><br>"
                  "<a href=""#Enter"">Commands in normal mode, maintaining vocabulary file</a><br>"
                  "<a href=""#Set"">Settings</a><br>"
                  "<a href=""#Options"">Command line options</a><br>"
                  "<a href=""#Dict"">A word about dict</a><br>"
                  "<a href=""#Examples"">Examples</a><br>"
                  "<a href=""#Faq"">FAQ</a><br>"
                  "<a href=""#Credits"">Credits</a><p>"

                  "<hr>"
                  "<a name=""Intro""><b><u>Introduction</u></b></a><p>"
                  "qvtwfc is a simple vocabulary trainer using flash cards.<p>"
                  "The main purpose to use qvtwfc is starting it with the '-t' switch to get into flash card (t=test) mode. "
                  "Here you will be asked a number of words randomly picked from the vocabulary file in use and then "
                  "the program just exits.<p>"
                  "It is meant to be started when you boot your PC to get the damn vocabularies into your head "
                  "by constant repeats but without taking too much of your time. You can set/unset starting a "
                  "flash card/test session every time you start a desktop session in the settings screen. "
                  "You can of course terminate a test session by clicking on the 'X' in the flash card window header, "
                  "if it is inconvenient at the moment.<p>"
                  "qvtwfc is not compatible to anything else, you have to enter every vocabulary pair by yourself.<p>"
                  "For that, start it without argument(s), to get into normal mode, in which you can maintain the "
                  "vocabulary files.<p>"
                  "There is no special naming rule for the vocabulary files, especially no specific ending was "
                  "'invented' to show the type of the files. Best you create a directory and store all vocabulary "
                  "files you create in there.<br> The number of word pairs per file can be huge, but it does not make "
                  "much sense to have such big files, as a word pair will be asked to seldom in that case. Better "
                  "create many different files, e. g. one per book you read or chapter you work on, so that these "
                  "words will be asked more often.<p>"
                  "The files themself are simple text files with<br>"
                  "one header line (which will be created automatically) of the format:<br>"
                  "Languages: original_language TAB translation_language<br>"
                  "followed by the word pairs, one per line, using a TAB as separator between original word, word type and translation:<br>"
                  "original_word/phrase TAB word type TAB translation<br>"
                  "TAB means exactly one TAB-stop = exactly 2 per line. So you can use any character but not TAB in the text. Close the "
                  "line by hitting 'Enter/Return'.<br>"
                  "For the language descriptions you can us free text, something like 'en' or 'de' however is short "
                  "and says it all.<br>"
                  "Word type can be v=verb, n=noun, adj=adjective, adv=adverb or o=other for phrase/term.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Start""><b><u>Starting</u></b></a><p>"
                  "qvtwfc can be started in two modes:<p>"
                  "qvtwfc = without command line arguments to get into normal mode.<br>"
                  "qvtwfc -t [more options] = with command line arguments to get into flash card/test mode.<p>"
                  "<b>qvtwfc</b><br>"
                  "- Maintain vocabulary file(s)<br>"
                  "When started without arguments you get the GUI in which you just opened this help screen. "
                  "Here you can create and maintain the vocabulary list files which will be used in flash card mode.<p>"
                  "- Settings dialog<br>"
                  "Further you can open a settings dialog and set some settings which will be described in the next chapter.<p>"
                  "<b>qvtwfc -t [options]</b><br>"
                  "Starting with the '-t' switch invokes the flash card/test mode.<br>"
                  "- options<br>"
                  "The number of word/phrase pairs to ask and the vocabulary file to use can be set in the settings screen "
                  "(in normal mode) or given as command line options. See the chapter <a href=""#Options"">Command line " "options</a> for more information on that.<br>"
                  "If no file name is specified either in the settings screen or via a command line option, the file last "
                  "worked with in normal mode will be used.<p>"
                  "- Toggle starting test when starting desktop session<br>"
                  "You can set/unset starting a test session (qvtwfc -t) without any more arguments every time you start "
                  "a desktop session in the settings dialog when in normal mode. See next chapter <a href=""#Set"">Settings</a>.<p>"
                  "So if this is the first time you use qvtwfc you do not have to worry about any settings, as long as "
                  "you have at least created one vocabulary file in normal mode.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Enter""><b><u>Commands in normal mode, maintaining vocabulary file</u></b></a><p>"
                  "<b>If the vocabulary list widget (the list on the left) has the focus:</b>"
                  "<table border=""1"" cellspacing=""0"" cellpadding=""5"">"
                  "<tr bgcolor=#dddddd>"
                  "  <td>Alt + d</td>"
                  "    <td>Will pop up a dialog to allow to delete the currently selected vocabulary pair.</td>"
                  "</tr>"
                  "<tr bgcolor=#eeeeee>"
                  "  <td>Right-click into a original word field </td>"
                  "    <td>Will pop up a dialog to allow to delete the currently selected vocabulary pair.</td>"
                  "</tr>"
                  "<tr bgcolor=#dddddd>"
                  "  <td>Right-click into a word type field </td>"
                  "    <td>Will open a dialog to allow to change the word type for this wordpair.</td>"
                  "</tr>"
                  "<tr bgcolor=#eeeeee>"
                  "  <td>Right-click into a translation word field </td>"
                  "    <td>Will open a dialog to allow to change the translation of this wordpair.</td>"
                  "</tr>"
                  "</table><p> <br>"
                  "<b>If the 'Save' button is enabled:</b>"
                  "<table border=""1"" cellspacing=""0"" cellpadding=""5"">"
                  "<tr bgcolor=#eeeeee>"
                  "  <td>F2 &#160; &#160;</td>"
                  "    <td>Save the word list.</td>"
                  "</tr>"
                  "</table><p>"
                  "<b>If the Dict search is enabled:</b>"
                  "<table border=""1"" cellspacing=""0"" cellpadding=""5"">"
                  "<tr bgcolor=#eeeeee>"
                  "  <td>F10 &#160; &#160;</td>"
                  "    <td>Clear Dict search widget.</td>"
                  "</tr>"
                  "</table><p>"
                  "<b>If the 'Add Entry' button in the 'Dict look up' input widget is enabled:</b>"
                  "<table border=""1"" cellspacing=""0"" cellpadding=""5"">"
                  "<tr bgcolor=#eeeeee>"
                  "  <td>F12 &#160; &#160;</td>"
                  "    <td>Add entry, wordpair, to vocabulary list. Same as clicking the 'Add Entry' button.</td>"
                  "</tr>"
                  "</table><p>"
                  "<b>Note:</b><br>"
                  "- You can edit the 'New entry' fields before adding the wordpair to the list, even, if was build "
                  "by selecting from the dict lookup results.<br>"
                  "- The word type will be automatically set, if the translation contains any of the keywords in {} brackets. "
                  "Otherwise you must select it manually from the combobox, if it is not a verb, the default.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Set""><b><u>Settings</u></b></a><p>"
                  "In normal mode you can open the settings dialog window by clicking the 'Settings' button.<br>"
                  "The following settings can be defined:<p>"
                  "For running in <b>normal mode:</b><br>"
                  "The TAB width in the text editor window. Set it wider than the length of the longest original "
                  "word/phrase, so that the translations all start in one column.<p>"
                  "For running in <b>flash card/test mode:</b><br>"
                  "The number of random word pairs to ask before exiting.<br>"
                  "Note, that you can override this setting by starting the program with the '-a xxx' switch, see the "
                  "chapter <a href=""#Options"">Command line options</a>.<p>"
                  "The vocabulary file to use in this mode. By default and if no file is defined here qvtwfc uses the "
                  "file you last worked with in normal mode.<br>"
                  "Note, that you can override this setting by providing a filename as a command line argument when "
                  "starting the program, see the chapter <a href=""#Options"">Command line options</a>.<p>"
                  "Last you can set/unset to start a flash card session every time you start a desktop session by "
                  "checking/unchecking the appropriate checkbox. Below it, you can see which desktops qvtwfc has detected "
                  "and whether a test session will run on them next time you start it.<br>"
                  "By checking this checkbox, a file named qvtwfc.desktop is copied into your desktop autostart folder.<br>"
                  "For KDE this is ~/.kde/Autostart, for xdg/freedesktop.org conform desktops like Gnome or xfce4 this "
                  "is ~/.config/autostart . The original is installed in /usr/share/applications .<br>"
                  "If your distribution uses another directory for the *.desktop files, this function will fail. If you "
                  "can locate the file qvtwfc.desktop yourself on your machine, you can manually copy it to the autostart "
                  "folder for the desktop you run.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Options""><b><u>Command line options</u></b></a><p>"
                  "<b>-t</b><br>"
                  "Start qvtwfc with the '-t' switch to get into flash card (t=test) mode.<p>"
                  "<b>-a #_word_pairs</b><br>"
                  "Ask the given number of word pairs. Overrides the number set in the settings screen in normal mode.<br>"
                  "This option only works with the '-t' switch.<p>"
                  "<b>filename</b><br>"
                  "Use the vocabulary file filename. Overrides the file setting from the settings screen in normal mode "
                  "and the default rule to use the file that was last worked on in normal mode, if none defined in the "
                  "settings screen.<br>"
                  "This option only works with the '-t' switch.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Dict""><b><u>A word about dict</u></b></a><p>"
                  "dict or the Dictionary Server Protocol (DICT) stands for free dictionaries following a TCP transaction protocol standard "
                  "according a 'Request for Comments #2229' from the 'Network Working Group' established in 1997. Copyright (C) The Internet Society (1997).<p>"
                  "The dictionaries do not need to deal with languages, but can be of other topics as well, like internet terms, law terms or whatever.<br>"
                  "qvtwfc, however, was designed to handle language dictionaries only, and in specific was tested with english/german and german/english dictionaries.<br>"
                  "But as UTF-8 should be supported, handling of other languages should work as well.<p>"
                  "The dict structure consist of a dict server, providing the dictionaries, and a client querying this server and its dictionaries.<br>"
                  "qvtwfc, this program, is a client, querying a server. You can set the server to query in qvtwfc's settings dialog.<br>"
                  "qvtwfc is intended to use a dict server on a local machine, the machine qvtwfc is running itself. Connecting to remote machines might work, "
                  "but was not tested.<p>"
                  "To set up a local dict server, like dictd, query your distribution's package manager, e. g. with the search word 'dict', "
                  "and install a dict server, like dictd, and the free available dictionaries for the languages you are interested in.<br>"
                  "You can later select the dictionaries which are used in qvtwfc among the installed once in the settings dialog, which you "
                  "can access via the Settings button at the bottom of the main widget/screen.<p>"
                  "If dict lookup is enabled (can be enabled/disabled in the afore mentioned settings dialog), at each startup qvtwfc checks if "
                  "the dict server and the formerly selected languages are available. The LED in the dict lookup area will tell you of the result "
                  "of this check. Hover over the LED to get the result in writing:<br>"
                  "grey: dict lookup disabled<br>"
                  "red : dict error, dict enabled but unavailable<br>"
                  "orange: not all selected dictionaries are avialable this time<br>"
                  "green: OK, all selected dictionaries are avialable<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Examples""><b><u>Examples</u></b></a><p>"
                  "qvtwfc<br>"
                  "This will start the program in normal mode, in which you can work on the vocabulary file(s) and define "
                  "settings as well.<p>"
                  "qvtwfc -t<br>"
                  "This will start the program in flash card (t=test) mode, using the vocabulary file last opened "
                  "in normal mode or the one set in the settings screen (if one specified) and asking the "
                  "number of words specified in the settings screen.<p>"
                  "qvtwfc -t /tmp/myvoc<br>"
                  "Starting in flash card mode as before but using the vocabulary file '/tmp/myvoc' .<p>"
                  "qvtwfc -t /tmp/myvoc -a 12<br>"
                  "As before, but asking 12 words only, regardless what is set in the settings screen.<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Faq""><b><u>FAQ</u></b></a><p>"
                  "No entries so far...<p>"
                  "<a href=""#Top"">back to top</a><p>"

                  "<hr>"
                  "<a name=""Credits""><b><u>Credits</u></b></a><p>"
                  "Icons where taken from the following themes made by the listed persons:<br>"
                  "From the Cristal SVG icon theme from KDE, made by Everaldo.com Design Studio: 'filenew.png', 'fileopen.png' and 'edit.png'.<br>"
                  "From the Gnome icon theme (around version 2.20): 'editdelete.png' as 'delete.png'.<p>"
                  "Program and Documentation Copyright 2007-2009 Helge Kortendieck alan@kasmithee.de .<p>"
                  "qvtwfc is released under GPL2 or later.<p>"
                  "<a href=""#Top"">back to top</a><p>"
                  "</body></html>" ) ;
      break;
    case 3:                                     // error text (txt, ASCII)
      // sText is already set from parent (qvtwfc.cpp/h)
      break;
  }
}


#include "abouthelp.moc"
