/***************************************************************************
   dict.h - handles communication with the dict server dictd via a QSocket
                         -------------------
   Copyright (C) 2006-2008 by Helge Kortendieck alan@kasmithee.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DICT_H_
#define DICT_H_

#include "structs.h"

#include <QAbstractSocket>
#include <QApplication>
#include <QList>
#include <QObject>
#include <QSettings>
#include <QString>
#include <QTcpSocket>
#include <QTextStream>
#include <QTimer>

#include <iostream>
using namespace std;

class dict : public QObject
{
  Q_OBJECT

public:
  dict( QObject *parent = 0 );
    ~dict();


  QString ssRead;           // debug all result lines...


public slots:
    /*$PUBLIC_SLOTS$*/

  signals:

protected:

private:
  bool bConnected;
  bool bConv;             // result of conversion to int: have server return code in response line
  bool bDictHaveThis;     // used during check if selected dictionary among dicts on server
  bool bDictLookUpWordBlocked; // do not start another word lookup before running finished
  bool bSomeDictsUnAvail; // flag whether some dictionaries from the config file are not available on the server

  int i;
  int i2;
  int iCom;               // current command: 0=show databases, 1=show info of database xy, 2=define <definition>, 3=match <definition>, 4=disconnect f dict server
  int iConState;          // connection to dict server state: 0=disconnected, 1=connected, 2=trying to connect
  int iD;
  int iD2;
  int iD3;
  int iDictPort;          // port of dict server
  int iDictQueryErr;      // error code returned by dict server during query
  int iDictToQuery;       // number of dictionary to query from list of selected dicts
  int iDS;                // set dict server state, 0=disabled, 1=no or none of selected dicts, 2=some selected avail, 3= all selected avail
  int iErr;
  int iRCode;             // response code from dict server
  int iRecon;             // when disconnecting, are we disconnecting=0 or reconnnecting=1
  int iResType;           // response type: 0=undefined_screwed up, 1=databases, 2=database info, 3=word match orig, 4=word match trans
  int iTask;              // current task: 0=idle, 1(2)3=connect to database n 1=check dicts avail, 2=word lookup, 3=send dict list to set dialog


  LookUp lookup;

  DictDef dictsEntry;             // entry in lists of dictionaries
  QList<DictDef> dictsInConf;     // dictionaries to be used acc. config
  QList<DictDef> dictsOnServer;   // dictionaries available on dict server
  QList<DictDef> dictsToQuery;    // dictionaries to query, on server available dicts of dictsInConf

  QList<LookUp> lookups;    // look up results from MATCH, corresponding dict, original and translation, will become lookup widget entry

  QString sD;             // dummy qstring, used only locally
  QString sDb;            // database to use for look up
  QString sDictd;         // dict server connect response
  QString sDictIp;        // IP address of dict server
  QString sErr;           // error string, non socket errors
  QString sLookUp;        // text to look up, received from qvtwfc

  QString sRead;          // answer from dict server
  QString sSend;          // command send to the router, full line incl. options

  QTcpSocket *sock;       // socket to talk to fli4l router

  QTimer *timerCom;       // 3 sec timer monitor whether a command to the router timed out = lost connection
  QTimer *timerCon;       // 2 sec timer to catch if a connection to dict server times out
//  QTimer *timerDisCon;    // 1 sec timer, to force disconnection after asking nicely timed out

private	slots:
  void init();                      // read settings, try to connect to dict server

  void checkDictsAvail();         // check if all dictionaries defined in config file are available on dict server

  void con();                     // connected to dict server
  void conConnected();            // receive connected to dict server
  void conConnectingTimedOut();   // connecting to dict server timed out
  void conReCon( const QString& sD, int iD);  // re-connect to dict server with new settings
  void conDisCon( int iRecon );   // disconnect from dict server
  void conDisConnected();         // receive disconnected from dict server
  void conDisConForce();          // hard close the socket

  void getDictsFromServer();      // get available databases/dictionaries from dict server
  void lookUpWordPre( const QString& sD );  // look up word preparation (might re-connect first)
  void lookUpWordLoopPre();       // prepare looping over the avialable dicts
  void lookUpWordLoopStart();     // look up word in 1st dictionary
  void lookUpWordLoopNext();      // look up word in next dictionary or send result
  void lookUpWord();              // look up word

//  void emitConErr();    // needed???????    // emit connection errors, if dict server reachable but access denied

  void settingsRead();        // read dict server relevant config settings, 1=re-read after changes in settings dialog (disconnect first!)
  void settingsDlgUpd();      // read dict server relevant config settings, 1=re-read after changes in settings dialog (disconnect first!)

  void sendCommand();         // send command to dict server
  void socketReadyRead();     // received answer from dict server

  void timerComStart();       // start 3 sec timer to allow command result to be received, else error
  void timerComStop();        // stop 3 sec timer, got result in time
  void timerComTimedOut();    // command send to the dict server timed out after 3 sec - unblock


  signals:
    void sigDicErrCon( int iTask, const QString& sErr );                      // error connecting to dict server: current task + error text
    void sigDicErrQuery( int iErr, const QString& sErr );                     // error returned by dict server while querying
    void sigDicSendMatches( const QList<LookUp>& lookups);                    // send matches from word look up
    void sigDicSendDictsOnServer( const QList<DictDef>& dictsOnServer, const QList<DictDef>& dictsToQuery ); // send all dictionaries available on the dict server and the once matching selection from settings dialog, used in settings dialog
    void sigDicSendDictsStatusUpd( int iDS, const QString& sErr );             // update status of available dictionaries (expecting iDSD > 0)
};

#endif
