/****************************************************************************
 **
 **  Copyright (C) 2007, 2008 Helge Kortendieck, alan@kasmithee.de
 **
 **  Main program, set number of words to test in test mode and
 **  handling vocabulary files: creating, changing...
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "qvtwfc.h"

Qvtwfc::Qvtwfc(QWidget *parent)
        : QWidget(parent)
{
  bAskSave = false;
  bDicObj = false;          // dict object not created so far
  bInit = true;
  bInitErr = false;         // flag whether an error occured while trying to load file from last session
  bNewFile = false;
  bOpenFile = false;        // flag whether a file should be opened after saving current
  bQuit = false;
  bDlgAh = false;           // not showing settings dialog at begin
  bDlgSet = false;          // not showing settings dialog at begin

  iDS = 0;                  // dict status: 0=disabled (grey), 1=server error (red), 2=selection partly available (orange), 3=OK, all selected dbs available (green)
  iInputFocus = 0;          // widget which has last got input focus: 0=undefined, 1=local seach, 2=local seach translation, 3=dict look up
  iPM = 0;
  iTwItemCurRow = -1;       // row of current tablewidgetitem
  iTwItemCurCol = -1;       // column of current tablewidgetitem, both will become 'PREV' after checks
  iTwItemPrevRow = -1;      // row of previous tablewidgetitem
  iTwItemPrevCol = -1;      // column of previous tablewidgetitem

  ui.setupUi(this);
  ui.pB_Save->setEnabled(false);

  defActions();             // define menu actions

  connect(ui.pB_About, SIGNAL( clicked() ), this, SLOT( about() ) );      // show about dialog
  connect(ui.pB_Help, SIGNAL( clicked() ), this, SLOT( help() ) );        // show online help
  connect(ui.pB_Quit, SIGNAL( clicked() ), this, SLOT( quitPrep() ) );    // quit app
  connect(ui.pB_Save, SIGNAL( clicked() ), this, SLOT( save() ) );        // save wordpair list
  connect(ui.pB_Settings, SIGNAL( clicked() ), this, SLOT( setDlgOpen() ) ); // open settings dialog
// wordpair file handling:
  connect(ui.pB_FileNew, SIGNAL( clicked() ), this, SLOT( fileNewClicked() ) );
  connect(ui.pB_FileOpen, SIGNAL( clicked() ), this, SLOT( fileOpenClicked() ) );
  connect(ui.lE_File, SIGNAL( returnPressed() ), this, SLOT( fileOpenPressed() ) );
// direct language and wordpair edit:
  connect(ui.lE_OrigLang, SIGNAL( textEdited( const QString& ) ), this, SLOT( askSaveTrue() ) );    // enalbe save button
  connect(ui.lE_TransLang, SIGNAL( textEdited(  const QString& ) ), this, SLOT( askSaveTrue() ) );  // enable save button
  connect(ui.tW_List->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timer5ChkHeadResized() )); // 5 sec delay t check col headers w resized->save
  connect(ui.tW_List, SIGNAL( currentCellChanged(int, int, int, int)), this, SLOT( twCellEntered(int, int, int, int)));// check if prev cell changed, if sort n enable save but
  connect(ui.tW_List, SIGNAL( customContextMenuRequested( const QPoint& ) ), this, SLOT( menuCalled( const QPoint& ))); // right-click menu on table widget - delete item, edit type or translation?
// search word in table:
  connect(ui.lE_Search, SIGNAL( textEdited( const QString& ) ), this, SLOT( localSearch( const QString& ) ) ); // check if an orig entry starts with this pattern
  connect(ui.lE_SearchTrans, SIGNAL( textEdited( const QString& ) ), this, SLOT( localSearchTrans( const QString& ) ) );  // check if a trans entry matches this pattern
  connect(ui.lE_SearchTrans, SIGNAL( returnPressed() ), this, SLOT( localSearchTransRP() ) ); // search local translation, return pressed, mark next match
// delete wordpair in table via mouse:
  ui.tW_List->setContextMenuPolicy(Qt::CustomContextMenu);                                    // needed for right-click context menu (del wordpair(s))

// dict lookup:
  connect(ui.lE_LookUp, SIGNAL( textEdited( const QString& ) ), this, SLOT( dictLookUp() ) );   // lookup field content changed, start new dict look up
  connect(ui.lE_LookUp, SIGNAL( textEdited( const QString& ) ), this, SLOT( localSearch( const QString& ) ) );  // show closes match of dict lookup in local wordpair list
  connect(ui.lE_LookUp, SIGNAL( textEdited( const QString& ) ), this, SLOT( dictTimerLookUpClearStart() ) );    // start 12sec timer to clear search input
  connect(ui.tW_DictRes, SIGNAL( itemChanged( QTableWidgetItem* ) ), this, SLOT( dictResTabClicked( QTableWidgetItem* ) ) );   // check if clicked item is in col 0
  connect(ui.tW_DictRes->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timerChkDictResHeadResized() )); // 5s delay t chk col headers w resized->save
  connect(ui.lE_ResOrig, SIGNAL( textEdited( const QString& ) ), this, SLOT( dictResAddButSet() ) );  // resulting new orig changed,check to enable add entry but
  connect(ui.lE_ResOrig, SIGNAL( textEdited( const QString& ) ), this, SLOT( localSearch( const QString& ) ) );   // new man entry, check if an orig entry matches this pattern
  connect(ui.cB_WordType, SIGNAL( activated( int ) ), this, SLOT( dictResTypSet() ) );                // word type for new entry was set by user, do not override it
  connect(ui.cB_WordType, SIGNAL( currentIndexChanged( int ) ), this, SLOT( dictResTypIdxChngd() ) ); // word type idx changed set focus to dict lookup, enable F12
  connect(ui.lE_ResTrans, SIGNAL( textEdited( const QString& ) ), this, SLOT( dictResAddButSet() ) ); // resulting new trans changed,chk to enable add entry but
  connect(ui.pB_AddDict, SIGNAL( clicked() ), this, SLOT( wpIns() ) );                                // add new resulting dict look up entry to wordpair list
  connect(QApplication::instance(), SIGNAL( focusChanged( QWidget*, QWidget* ) ), this, SLOT( widgetChgd( QWidget*, QWidget* ) ) ); // new widget got focus, upd input after save if req.

  rwf = new rw();                           // setup read/write files qobject
  connect( rwf, SIGNAL( sigReadAbort( const QString&, const QString&, int ) ), this, SLOT( readAbort( const QString&, const QString&, int ) ) ); // fatal err while opening file
  connect( rwf, SIGNAL( sigReadWErr( const QString&, const QString&, int, const QString&, const QString& ) ), this, SLOT( readWErr( const QString&, const QString&, int, const QString&, const QString& ) ) );                                  // errors while reading file
  connect( rwf, SIGNAL( sigReadOk( const QString&, const QString& ) ), this, SLOT( readOk( const QString&, const QString& ) ) );  // finished reading file, no errors
  connect( rwf, SIGNAL( sigWriteAbort( const QString&, const QString&, int ) ), this, SLOT( writeAbort( const QString&, const QString&, int ) ) ); // finished writing file
  connect( rwf, SIGNAL( sigWriteWErr( const QString&, const QString&, int ) ), this, SLOT( writeWErr( const QString&, const QString&, int ) ) );   // error while writing file
  connect( rwf, SIGNAL( sigWriteOk() ), this, SLOT( writeOk() ) );  // finished writing file

  uia.setupUi( &wAsk );                     // dialog asking for languages when creating a new vocabulary file
  connect(uia.pB_Set, SIGNAL( clicked() ), this, SLOT( askLangSet() ) );
  connect(uia.pB_Cancel, SIGNAL( clicked() ), this, SLOT( askLangCancel() ) );

  uiwt.setupUi( &wEdwt );                   // dialog to edit word type of a wordpair
  connect(uiwt.pB_Upd, SIGNAL( clicked() ), this, SLOT( wpEdTypeUpd() ) );
  connect(uiwt.pB_Cancel, SIGNAL( clicked() ), this, SLOT( wpEdTypeCancel() ) );

  uiwtrans.setupUi( &wEdwtrans );                   // dialog to edit word type of a wordpair
  connect(uiwtrans.pB_Upd, SIGNAL( clicked() ), this, SLOT( wpEdTransUpd() ) );
  connect(uiwtrans.pB_Cancel, SIGNAL( clicked() ), this, SLOT( wpEdTransCancel() ) );

  timerSaveWordList = new QTimer(this);         // 5min timer to save word list, started if word list changed and not already active
  timerSaveWordList->setSingleShot( true );
  connect(timerSaveWordList, SIGNAL(timeout()), this, SLOT( save() ) );

  settingsRead();

  timer5 = new QTimer(this);                    // 5 sec timer to limit checks if the table column headers where resized
  timer5->setSingleShot( true );
  connect(timer5, SIGNAL(timeout()), this, SLOT( chkHeadResized() ) );    // check if col headers where resized -> save

  timer5Dict = new QTimer(this);                // 5 sec timer to unblock further dict queries when no answer was received, probably lost dict server
  connect(timer5Dict, SIGNAL(timeout()), this, SLOT( dictTimer5DictTimedOut() ) );

  timerDictResHeadReSized = new QTimer(this);                    // 5 sec timer to limit checks if the table column headers where resized
  timerDictResHeadReSized->setSingleShot( true );
  connect(timerDictResHeadReSized, SIGNAL(timeout()), this, SLOT( chkDictResHeadResized() ) );    // check if col headers where resized -> save

  timerDictLookUpClear = new QTimer(this);      // timer to clear dict lookup input widget after 12 sec
  timerDictLookUpClear->setSingleShot( true );
  connect(timerDictLookUpClear, SIGNAL(timeout()), this, SLOT( dictAutoClearLookUpInput() ) );

  menuDelWp = new QMenu;                        // mini menu to delete item after right mouse click in table
  menuDelWp->clear();
  menuDelWp->addAction( actDelWp );                 // delete wordpair

  menuEditType = new QMenu;                     // mini menu to edit word type after right mouse click in table
  menuEditType->clear();
  menuEditType->addAction( actEditType );           // edit word type

  menuEditTrans = new QMenu;                    // mini menu to edit translation after right mouse click in table
  menuEditTrans->clear();
  menuEditTrans->addAction( actEditTrans );         // edit translation

  initDelay();                                  // short timer to allow GUI to come up before proceeding...
}

void Qvtwfc::settingsRead()
{
  QSettings settings("qvtwfc", "qvtwfc");

  settings.beginGroup( "mainwindow" );
    QPoint pos = settings.value( "pos", QPoint(0, 0) ).toPoint();
    QSize size = settings.value( "size", QSize(670, 430) ).toSize();
    ui.splitterH->restoreState(settings.value( "splitterHori" ).toByteArray());
    ui.splitterV->restoreState(settings.value( "splitterVert" ).toByteArray());
    iFontSize =  settings.value( "fontsize", 11 ).toInt();
    iCol0Width = settings.value( "col0width", 90 ).toInt();
    iCol1Width = settings.value( "col1width", 15 ).toInt();
    iCol2Width = settings.value( "col2width", 200 ).toInt();
    sFile = settings.value( "file", "/" ).toString();
  settings.endGroup();
  settings.beginGroup( "dict" );
    bDictUse = settings.value( "dictenable", "false" ).toBool();
    iColRes0Width = settings.value( "colres0width", 70 ).toInt();    // width of translation result column
    iColRes1Width = settings.value( "colres1width", 70 ).toInt();    // width of orig match result column
    iColRes2Width = settings.value( "colres2width", 70 ).toInt();    // width of dictionary result column
  settings.endGroup();

  setFontSize();
  resize(size);
  move(pos);

  ui.lE_File->setText( sFile );
}

void Qvtwfc::settingsReRead()               // received request from settings dialog to re-read settings
{
  settingsRead();
  if( bDicObj == true ) {                       // let dict object re-read settings too
    emit sigQvtDicReReadSet();
  }
}

void Qvtwfc::settingsWrite()
{
  QSettings settings("qvtwfc", "qvtwfc");

  settings.beginGroup( "mainwindow" );
    settings.setValue( "pos", pos() );
    settings.setValue( "size", size() );
    settings.setValue( "splitterHori", ui.splitterH->saveState());
    settings.setValue( "splitterVert", ui.splitterV->saveState());
    settings.setValue( "col0width", ui.tW_List->columnWidth( 0 ) );
    settings.setValue( "col1width", ui.tW_List->columnWidth( 1 ) );
    settings.setValue( "col2width", ui.tW_List->columnWidth( 2 ) );
    settings.setValue( "file", sNf );
  settings.endGroup();
  settings.beginGroup( "dict" );
    settings.setValue( "colres0width", iColRes0Width );   // translation column, use this and not take actual, as the column might not set at the moment
    settings.setValue( "colres1width", iColRes1Width );   // orig match,   "
    settings.setValue( "colres2width", iColRes2Width );   // dictionary   "
  settings.endGroup();

  settings.sync();

  if( bQuit ) {
    if( bSaveFirst ) {
      writePrep();
    } else {
      quitDelay();
    }
  }
}

void Qvtwfc::defActions()
{
  actDelWp = new QAction( QIcon( "/usr/share/apps/qvtwfc/icons/32x32/actions/delete.png" ), tr( "&Delete selected wordpair" ), this);
  actDelWp->setStatusTip( tr( "Delete the currently selected wordpair" ) );
  connect( actDelWp, SIGNAL( triggered() ), this, SLOT( wpDel() ) );

  actEditType = new QAction( QIcon( "/usr/share/apps/qvtwfc/icons/32x32/actions/edit.png" ), tr( "&Edit word type" ), this);
  actEditType->setStatusTip( tr( "Edit the type of the selected wordpair" ) );
  connect( actEditType, SIGNAL( triggered() ), this, SLOT( wpEdType() ) );

  actEditTrans = new QAction( QIcon( "/usr/share/apps/qvtwfc/icons/32x32/actions/edit.png" ), tr( "&Edit translation" ), this);
  actEditTrans->setStatusTip( tr( "Edit the translation of the selected wordpair" ) );
  connect( actEditTrans, SIGNAL( triggered() ), this, SLOT( wpEdTrans() ) );
}

void Qvtwfc::setFontSize()                  // set font size of main widget excl. buttons
{
  QFont font;
    font.setPointSize( iFontSize );
    ui.splitterH->setFont( font );
    ui.f_New->setFont( font );
    ui.f_Search->setFont( font );
    twPop();                                    // required for updating width of vertical headers (row #s)
}

void Qvtwfc::about()
{
  ahDlgCreate( 1 );
}

void Qvtwfc::help()
{
  ahDlgCreate( 2 );
}

void Qvtwfc::quitPrep()                     // quit clicked clean up
{
  bQuit = true;
  bSaveFirst = false;
  if(!QMessageBox::question(0, tr( "Quit ?" ), 
                               tr( "\nSure to quit ?\n" ), 
                               tr( "Yes, &Quit" ), tr( "No, &Cancel" ), 0, 1 )) {
    if( bAskSave ) {
      if(!QMessageBox::question( 0, tr( "Save File ?" ), 
                                    tr( "\nThe vocabulary file was changed.\n"
                                        "Would you like to save it before exiting ?\n \n" ), 
                                    tr( "Yes, &Save" ), tr( "No, Just &Quit" ), 0, 1 )) {
        bSaveFirst = true;
      }
    }
    settingsWrite();
  }
}

void Qvtwfc::quitDelay()                    // delay to allow clean up - needed ????????????????????
{
  QTimer::singleShot(500, this, SLOT( quitDo() ) );
}

void Qvtwfc::quitDo()                       // finally quit the program
{
  exit(0);
}


void Qvtwfc::save()                         // save clicked
{
  writePrep();
  switch ( iInputFocus ) {                      // set focus back to input widget which had it last before clicking the save button
    case 1:                                     // local original search
      ui.lE_Search->setFocus();
      break;
    case 2:                                     // local translation search
      ui.lE_SearchTrans->setFocus();
      break;
    case 3:                                     // dict look up
      ui.lE_LookUp->setFocus();
      break;
  }
}

void Qvtwfc::askSaveTrue()                  // set bAskSave true and enable save button
{
  bAskSave = true;
  ui.pB_Save->setEnabled( true );
  if( timerSaveWordList->isActive() == false ) {
    timerSaveWordList->start( 300000 );
  }
}

void Qvtwfc::askSaveFalse()                 // set bAskSave false and disable save button
{
  bAskSave = false;
  ui.pB_Save->setEnabled( false );
  if( timerSaveWordList->isActive() == true ) {
    timerSaveWordList->stop();
  }
}

void Qvtwfc::initDelay()
{
  QTimer::singleShot(200, this, SLOT( init() ) );
}

void Qvtwfc::init()
{
  bInitErr = false;                             // flag whether there was an error during init, if, show message at end
  iDS = 0;                                      // dict status: 0=disabled=grey, 1=server error=red, 2=selection partly avail=orange, 3=all selected di avail=green
  dictUpdIcon();                                // set dict availability icon
  if( bDictUse == true ) {                      // try to connect to dict server and get list of available databases
    dictCreate();                               // enable dict server search, create dic object
  }
  sOf = ui.lE_File->text();
  if( sOf == "" or sOf == "/" ) {               // don't try to load a file if no chance of valid file name
    initEnd();
  } else {
    fileChkPrep();
  }
}

void Qvtwfc::initEnd()
{
  if( bInitErr ) {
    QMessageBox::information( this, tr( "Error during initalization !"), 
                                    tr( "Could not load file from last session!\n"
                                        "Re-setted to '/'.\n \n" ) );
    ui.lE_File->setText( "/" );
    bInitErr = false;
  }
  bInit = false;
  askSaveFalse();
}

void Qvtwfc::timer5ChkHeadResized()
{
  if( timer5->isActive() == false ) {
    timer5->start(5000);
  }
}

void Qvtwfc::widgetChgd( QWidget* qwD, QWidget* qwD2 )
{
  if( qwD2 == ui.lE_Search ) {
    iInputFocus = 1;
  } else if( qwD2 == ui.lE_SearchTrans ) {
    iInputFocus = 2;
  } else if( qwD2 == ui.lE_LookUp ) {
    iInputFocus = 3;
  }
}

void Qvtwfc::chkHeadResized()               // check if any header width of tW_List has been resized
{
  if( iCol0Width != ui.tW_List->columnWidth( 0 ) or iCol1Width != ui.tW_List->columnWidth( 1 ) or iCol2Width != ui.tW_List->columnWidth( 2 ) ) {
    if( ui.tW_List->columnCount() == 3 ) {             // cannot be sure, that the columns are set at this moment !
      iCol0Width = ui.tW_List->columnWidth( 0 );
      iCol1Width = ui.tW_List->columnWidth( 1 );
      iCol2Width = ui.tW_List->columnWidth( 2 );
      settingsWrite();
    }
  }
}

void Qvtwfc::timerChkDictResHeadResized()   // 5 sec delay timer to check if column header widths of dict result widget have been changed
{
  if( timerDictResHeadReSized->isActive() == false ) {
    timerDictResHeadReSized->start(3000);
  }
}

void Qvtwfc::chkDictResHeadResized()        // check if any header width of tW_DictRes has been resized
{
  if(iColRes0Width != ui.tW_DictRes->columnWidth( 0 ) or iColRes1Width != ui.tW_DictRes->columnWidth( 1 ) or iColRes2Width != ui.tW_DictRes->columnWidth( 2 ) ) {
    if( ui.tW_DictRes->columnCount() == 3 ) {             // cannot be sure, that the columns are set at this moment !
      iColRes0Width = ui.tW_DictRes->columnWidth( 0 );
      iColRes1Width = ui.tW_DictRes->columnWidth( 1 );
      iColRes2Width = ui.tW_DictRes->columnWidth( 2 );
    }
    settingsWrite();
  }
}


// ****************************************************************
// **  settings dialog functions/slots                           **
// ****************************************************************
void Qvtwfc::setDlgOpen()                   // button 'settings' clicked, show settings dialog
{
  if( bDlgSet == false ) {
    bDlgSet = true;
//    cout << "setDlgOpen(): reached ! " << endl;                                  // *****  debug  *****
    set = new settings();                       // settings dialog
    connect( this, SIGNAL( sigQvtSendDictsToSetDlg( const QList<DictDef>&, const QList<DictDef>& ) ), set, SLOT( setDlgReceiveAvailDicts( const QList<DictDef>&, const QList<DictDef>& ) ) ); // request to reread settings,could be changed via settings dialog
    connect( set, SIGNAL( sigSetDlgGetDicts( const QString&, int ) ), this, SLOT( dictReCon( const QString&, int ) ) ); // dict IP & dict Port
    connect( set, SIGNAL( sigSetDlgClose() ), this, SLOT( setDlgClose() ) );              // settings dialog closed - destroy
    connect( set, SIGNAL( sigSetDlgReReadSettings() ), this, SLOT( settingsReRead() ) );  //receive look up matches
  }
}

void Qvtwfc::setDlgDictErr()                // error while querying the dict server
{
//  QMessageBox::warning( this, tr( "Error - with dict server !" ), sDictTT + tr( "\nCheck the dict server settings and retry." ) );
}

void Qvtwfc::setDlgClose()                  // close settings dialog
{
  bDlgSet = false;
  set->deleteLater();
}


// ****************************************************************
// **  about/help dialog functions/slots                         **
// ****************************************************************
void Qvtwfc::ahDlgCreate( int iD )            // open about/help dialog, 1=about, 2=help
{
  if( bDlgAh == false ) {
    bDlgAh = true;
    iAhType = iD;
    ahw = new aboutHelp();
    connect( this, SIGNAL( sigAhDlg( int ) ), ahw, SLOT( showType( int ) ) );
    connect( this, SIGNAL( sigAhDlgClose() ), ahw, SLOT( closeDlg() ) );
    connect( ahw, SIGNAL( sigAhClosed() ), this, SLOT( ahDlgClosed() ) );
    emit sigAhDlg( iAhType );
  } else {
    emit sigAhDlgClose();
  }
}

void Qvtwfc::ahDlgClosed()                  // about/help dialog closed, delete
{
  bDlgAh = false;
  ahw->deleteLater();
}


// ****************************************************************
// **  file new and file open functions/slots                    **
// ****************************************************************
// ***************  file new functions/slots  ***************
void Qvtwfc::fileNewClicked()               // icon to create new vocabulary file clicked
{
  if( bAskSave ) {                              // ask to save current file first
    bNewFile = true;
    writePrep();
  } else {
    fileCreateDialog();
  }
}

void Qvtwfc::fileCreateDialog()             // dialog to create new vocabulary file
{
  fileGetStartDir();
  sFile = QFileDialog::getSaveFileName( this, tr( "Create new vocabulary file" ), sDir, "" );
  if( sFile == "" ) {                           // choosed cancel
    return;
  } else {
    QFileInfo fi2( sFile );                     // check new file name
    if( fi2.exists() ) {
      if(!QMessageBox::question(0, tr( "File already exists !" ), 
                                   tr( "A file with this name already exists !\n"
                                       "Do you want to overwrite it ?" ), 
                                   tr( "Yes, &Continue" ), tr( "No, &Abort" ), 0, 1 )) {
      } else {
        return;
      }
    }
  }
  askSaveTrue();
  ui.lE_File->setText( sFile );                 // set new file name
  askLangShow();
}


void Qvtwfc::fileGetStartDir()              // get start dir for file new/open dialog
{
  QFileInfo fi( ui.lE_File->text() );           // get start dir for dialog
  sDir = fi.absolutePath();
  QFileInfo fi2( sDir );
  if( fi2.exists() == false or fi2.isWritable() == false ) {
    sDir = QDir::homePath();
  }
}


// ***************  file open functions/slots  ***************
void Qvtwfc::fileOpenClicked()              // icon to open file dialog clicked
{
  fileOpenDialog();
}


void Qvtwfc::fileOpenPressed()              // return in file qlineedit pressed
{
    sOf = ui.lE_File->text();
    fileChkPrep();
}


void Qvtwfc::fileOpenDialog()
{
  fileGetStartDir();
  sOf = QFileDialog::getOpenFileName(this, tr("Open vocabulary file"), sDir, "" );

  if( sOf == "" ) {                             // choosed cancel
    return;
  } else {
    fileChkPrep();
  }
}

void Qvtwfc::fileChkPrep()
{
  bOpenFile = false;
  if( bAskSave ) {
    bOpenFile = true;
    writePrep();
  } else {
    sNf = sOf;
    rwf->readFileChk( sNf, 0 );                 // continue, check file (can update?) and try to open, 0 = normal mode
  }
}

void Qvtwfc::readAbort( const QString& sErrTit, const QString& sErrTxt, int iErr )  // fatal error while opening file
{
  ahw->sTitle = "qvtwfc - error " + QString::number( iErr) + ": " + sErrTit;
  ahw->sText = sErrTxt;
  emit sigDlgErr( iFontSize );                  // show errors, then stop
  if( iPM == 0 ) {                              // normal mode
    if( bInit and iErr != 9 ) {                 // iErr == 9 => file is empty, do not re-set to /
      bInitErr = true;
      initEnd();
      return;
    }
  }
}

void Qvtwfc::readWErr( const QString& sErrTit, const QString& sErrTxt, int iErr, const QString& sD1, const QString& sD2 ) // finished reading file, no fatal errors
{
  ahw->sTitle = "qvtwfc - error " + QString::number( iErr) + ": " + sErrTit;
  ahw->sText = sErrTxt;
  emit sigDlgErr( iFontSize );                  // show errors, then continue
  if( iPM == 0 ) {                              // normal mode
    sO = sD1;
    sT = sD2;
    wordpair = rwf->wordpair;
    iWpCnt = wordpair.count();
    ui.lE_File->setText( sNf );
    twPop();
  }
}

void Qvtwfc::readOk( const QString& sD1, const QString & sD2 ) // finished reading file, no errors
{
  if( iPM == 0 ) {                              // normal mode
    sO = sD1;
    sT = sD2;
    wordpair = rwf->wordpair;
    iWpCnt = wordpair.count();
    ui.lE_File->setText( sNf );
    twPop();
  }
}


// ****************************************************************
// **  ask language functions/slots                              **
// ****************************************************************
void Qvtwfc::askLangShow()                  // ask original and translation language when creating new voc file
{
  wAsk.show();
}

void Qvtwfc::askLangSet()                   // use original and translation language when creating new voc file
{
  sD = uia.lE_OrigLang->text();
  sD2 = uia.lE_TransLang->text();
  if( sD.isEmpty() ) {
    sD = "en";
  }
  if( sD2.isEmpty() ) {
    sD2 = "de";
  }
  ui.lE_OrigLang->setText( sD );
  ui.lE_TransLang->setText( sD2 );
  askSaveTrue();
  askLangCancel();
}

void Qvtwfc::askLangCancel()                // canceled original and translation language dialog
{
  wAsk.hide();
}


// ****************************************************************
// **  tablewidget functions/slots                               **
// ****************************************************************
void Qvtwfc::twPop()                        // populate tablewidget with word pairs
{
  disconnect(ui.tW_List->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timer5ChkHeadResized() )); // 5 sec delay t check col headers w
  if( wordpair.count() > 0 ) {
    wplD = wordpair;                            // **********  start sorting wordpairs  **********
    wordpair.clear();
    wordpair.append( wplD.at( 0 ) );            // init

    for( i = 1; i < wplD.count(); ++i ) {       // iterate over wordpairs
      sD = wplD.at( i ).orig;
      bIns = false;

      for( i2 = wordpair.count() - 1; i2 > -1; i2-- ) {
        if( sD > wordpair.at( i2 ).orig ) {
          wordpair.insert( i2 + 1, wplD.at( i ) );
          bIns = true;

          break;
        }
      }

      if( bIns == false ) {                     // **************** needed ???? *****************
        wordpair.insert( 0, wplD.at( i ) );
      }
    }
  }                                             // **********  end sorting  ***********

  ui.lE_OrigLang->setText( sO );
  ui.lE_TransLang->setText( sT );
  ui.tW_List->setRowCount( wordpair.size() );
  ui.tW_List->setColumnCount( 3 );
  ui.tW_List->setToolTip( tr( "word pairs" ) );
  slD.clear();                                      // set table widget headers
  slD.append( tr( "Original" ) );
  slD.append( tr( "Type" ) );
  slD.append( tr( "Translation" ) );
  ui.tW_List->setHorizontalHeaderLabels( slD );

  for ( i = 0; i < wordpair.count(); ++i ) {
    sD = wordpair.at( i ).orig;
    sD2 = wordpair.at( i ).type;
    sD3 = wordpair.at( i ).trans;
    QTableWidgetItem *newItem = new QTableWidgetItem( sD );             // original word
    ui.tW_List->setItem( i, 0, newItem);
    QTableWidgetItem *newItem2 = new QTableWidgetItem( sD2 );           // word type
    ui.tW_List->setItem( i, 1, newItem2);
    QTableWidgetItem *newItem3 = new QTableWidgetItem( sD3 );           // translation
    ui.tW_List->setItem( i, 2, newItem3);
  }

  ui.tW_List->setColumnWidth( 0, iCol0Width );
  ui.tW_List->setColumnWidth( 1, iCol1Width );
  ui.tW_List->setColumnWidth( 2, iCol2Width );

  connect(ui.tW_List->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timer5ChkHeadResized() )); // 5 sec delay t check col headers w

  if( bInit ) {
    initEnd();
  }
}

void Qvtwfc::twCellEntered( int iD, int iD2, int iD3, int iD4 ) // check, whether prev cell was changed, if so call sort n activate save button. iD=row, iD2=column
{
  iTwItemCurRow = iD;
  iTwItemCurCol = iD2;
  sTwItemCur = ui.tW_List->item( iD, iD2 )->text();   // current twitem (will become sTwItemPrevOnEntry after checks)

  if( iTwItemPrevRow > -1 and iTwItemPrevCol > -1 ) { // basically always true after first change. if change, sort and enable save button
    if( ui.tW_List->item( iTwItemPrevRow, iTwItemPrevCol )->text() != sTwItemPrevOnEntry ) { // last cell was changed, enable save button
      askSaveTrue();
    }
  }

  iTwItemPrevRow = iTwItemCurRow;           // now current is row of previous tablewidgetitem
  iTwItemPrevCol = iTwItemCurCol;           // now current is column of previous tablewidgetitem
  sTwItemPrevOnEntry = sTwItemCur;          // now current is content of previous tablewidgetitem on entry

  ui.tW_List->sortItems( 0, Qt::AscendingOrder );
}


// **************************************************************************
// **  write file                                                          **
// **************************************************************************
void Qvtwfc::writePrep()                    // create QStringlist with valid QTextEdit widget entries
{
  bWriteErr = false;
  sWriteErr = "";

  ui.tW_List->sortItems( 0, Qt::AscendingOrder );
  wordpair.clear();
  for( i = 0; i < ui.tW_List->rowCount(); ++i ) {
    wpD.orig = ui.tW_List->item( i, 0 )->text();
    wpD.type = ui.tW_List->item( i, 1 )->text();
    wpD.trans = ui.tW_List->item( i, 2 )->text();
    wordpair.append( wpD );
  }

  sD1 = ui.lE_OrigLang->text();             // set languages as header
  if( sD1 == "" ) {
    sD1 = tr( "Err" );
  }
  sD2 = ui.lE_TransLang->text();
  if( sD2 == "" ) {
    sD2 = tr( "Err" );
  }
  sD = "Languages:" + sD1 + "\t" + sD2;

  sNf = ui.lE_File->text();
  if( sNf != "" ) {
    rwf->wordpair = wordpair;
    rwf->writeFileChk( sNf, sD1, sD2 );
  } else {
    writeOk();
  }
}

void Qvtwfc::writeAbort( const QString& sErrTit, const QString& sErrTxt, int iErr )
{
  QMessageBox::information( this, sErrTit, sErrTxt );
  sQuitTit = tr( "without" );
  sQuitTxt = tr( "all" );
  bWriteErr = true;
  writeOk();
}

void Qvtwfc::writeWErr( const QString& sErrTit, const QString& sErrTxt, int iErr )
{
  QMessageBox::warning( this, QString::number( iErr) + ": " + sErrTit, sErrTxt );
  sQuitTit = tr( "despite incomplete" );
  sQuitTxt = tr( "some" );
  bWriteErr = true;
  writeOk();
}

void Qvtwfc::writeOk()
{
  askSaveFalse();
  if( bWriteErr == true ) {
    bWriteErr = false;
    askSaveTrue();

    if( bQuit ) {
      if(!QMessageBox::question(0, tr( "Quitting %1 save ?" ).arg( sQuitTit ), 
                                   tr( "\nDo you still want to exit and loose %1 changes to the file or\n"
                                       "do you want to abort and try to fix the problem ?\n" ).arg( sQuitTxt ), 
                                   tr( "Continue, &Quit" ), tr( "&Abort, Do Not Quit" ), 0, 1 )) {
        quitDelay();
      } else {
        bQuit = false;
        bSaveFirst = false;
      }
    }

    if( bNewFile ) {
      if(!QMessageBox::question(0, tr( "New File %1 save ?" ).arg( sQuitTit ), 
                                   tr( "\nDo you still want to create the new file and loose "
                                       "%1 changes in the current file or\n"
                                       "do you want to abort and try to fix the problem ?\n" ).arg( sQuitTxt ), 
                                   tr( "Continue, &Create New" ), tr( "&Abort, I Fix First" ), 0, 1 )) {
        fileCreateDialog();
      } else {
        bNewFile = false;
      }
    }

    if( bOpenFile ) {
      if(!QMessageBox::question(0, tr( "Open File %1 save ?" ).arg( sQuitTit ), 
                                   tr( "\nDo you still want to open the new file and loose "
                                       "%1 changes to the current file or\n"
                                       "do you want to abort and try to fix the problem ?\n" ).arg( sQuitTxt ), 
                                   tr( "Continue, &Open New" ), tr( "&Abort, I Fix First" ), 0, 1 )) {
        sNf = sOf;
        rwf->readFileChk( sNf, 0 );                        // continue, check file (can update?) and try to open
      } else {
        bOpenFile = false;
      }
    }

  } else {
    if( bQuit ) {
      quitDelay();
    } else if( bNewFile ) {
      fileCreateDialog();
    }
  }
}


// **************************************************************************
// **  dict server handling                                                **
// **************************************************************************
void Qvtwfc::dictCreate()                   // enable dict server search, create dic object
{
  if( bDicObj == false ) {
    bDicObj = true;                             // dict object not created so far
    dic = new dict();                           // handling of dict server
    connect( this, SIGNAL( sigQvtDicLookUp( const QString& ) ), dic, SLOT( lookUpWordPre( const QString& ) ) );     // look up word
    connect( this, SIGNAL( sigQvtDicDisCon( int ) ), dic, SLOT( conDisCon( int ) ) );                               // disconnect - disabling dict lookup/closing
    connect( this, SIGNAL( sigQvtDicReCon( const QString&, int ) ), dic, SLOT( conReCon( const QString&, int ) ) ); // re-connect to dict server with new settings
    connect( this, SIGNAL( sigQvtDicReReadSet() ), dic, SLOT( settingsDlgUpd( ) ) );                                // re-read settings, triggered by settings dlg
    connect( dic, SIGNAL( sigDicErrCon( int, const QString& ) ), this, SLOT( dictConErr( int, const QString& ) ) );     // dict server error, cannot connect
    connect( dic, SIGNAL( sigDicErrQuery( int, const QString& ) ), this, SLOT( dictQueryErr( int, const QString& ) ) ); // dict server error during query
    connect( dic, SIGNAL( sigDicSendMatches( const QList<LookUp>& ) ), this, SLOT( dictReceiveMatches( const QList<LookUp>& ) ) ); //receive look up matches
    connect( dic, SIGNAL( sigDicSendDictsStatusUpd( int, const QString& ) ), this, SLOT( dictDictsStatusUpd( int, const QString& ) ) );  // upd stat of avail dicts
    connect( dic, SIGNAL( sigDicSendDictsOnServer( const QList<DictDef>&, const QList<DictDef>& ) ), this, SLOT( dictReceiveDictsOnServer( const QList<DictDef>&, const QList<DictDef>& ) ) );  // receive dicts on server and matches acc. config file, used in settings dialog
  }
}

void Qvtwfc::dictConErr( int iDictErr, const QString& sD )  // error connecting to dict server, 1=starting, 2=word lookup, 2=getting list for set dlg
{
//QMessageBox::information( this, "debug - in dictGetMatches !", "reached" );   // *****   debug   *****
  sDictTT = tr( "Dict server unavailable: error code %1 : " ).arg( QString::number( iDictErr ) ) + sD; // dict server LED tooltip
  iDS = 0;                                      // dict status: 0=disabled, 1=server error, 2=selection partly available, 3=OK, all selected dbs available
  dictUpdIcon();
  bDictUse = false;
  if( iDictErr == 3 ) {                         // dict server queried from settings dialog
    setDlgDictErr();
  } else {
    QMessageBox::information( this, tr( "dict error !" ), tr( "Dict server unavailable:\nerror code %1 :\n" ).arg( QString::number( iDictErr ) ) + sD );
    dictDisable();
  }
}

void Qvtwfc::dictDictsStatusUpd( int iDSD, const QString& sErr ) // update status of available dictionaries (expecting iDSD > 0)
{
  iDS = iDSD;                               // dict status: 0=disabled, 1=no or none requested dict avail, 2=selection partly available, 3=all selected dbs avail
  if( iDS == 0 or iDS == 1 ) {
    QMessageBox::warning( this, tr( "dict warning !" ),
                                tr( "Status code: %1\n" ).arg( iDS ) + sErr );
    iDS = 0;
    dictDisable();
  } else if( iDS == 2 or iDS == 3 ) {
    if( iDS == 2 ) {
      QMessageBox::information( this, tr( "dict information !" ),
                                      tr( "Status code: %1\n" ).arg( iDS ) + sErr );
    }
    dictSearchEnable();
    dictUpdIcon();
  }
}

void Qvtwfc::dictQueryErr( int iDictErr, const QString& sDictErr )  // error returned by the dict server + warn if some selected dictionaries are unavailable
{
  QMessageBox::information( this, tr( "dict error !" ),
                                  tr( "The dict server returned error code: %1\n"
                                      "Error text: %2\n"
                                      "In case this error is currently not handled, "
                                      "see dictd documentation for the return/error codes or, "
                                      "as a first workaround, try to disable and then re-enable "
                                      "dict look up in the settings dialog." ).arg( iDictErr ).arg( sDictErr ) );
}

void Qvtwfc::dictReCon( const QString& sD, int iD ) // query dict server after dicts button in settings dialog clicked
{
  if( bDicObj == false ) {                      // dict object not created so far ) {
    dictCreate();
  }
//cout << "dictSettingsQuery(): reached ! " << endl;                                  // *****  debug  *****
  emit sigQvtDicReCon( sD, iD );
}

void Qvtwfc::dictReceiveDictsOnServer( const QList<DictDef>& dictsD, const QList<DictDef>& dictsD2 ) // receive all dicts avail on dict server n matches f config file, used in settings dialog
{
//cout << "dictReceiveDictsnServer(): reached ! " << endl;                                  // *****  debug  *****
  emit sigQvtSendDictsToSetDlg( dictsD, dictsD2 );
}

void Qvtwfc::dictDisable()                  // disable dict search, could not connect to server or no dicts on server
{
  sigQvtDicDisCon( 0 );                         // disconnect - disabling dict lookup/closing
  dic->deleteLater();
  bDicObj = false;                              // dict object not created so far
  iDS = 0;
  dictSearchDisable();
}


// **************************************************************************
// **  dict search widget handling                                         **
// **************************************************************************
void Qvtwfc::dictSearchEnable()             // enable dict search frame, have some configured dicts
{
  if( ui.f_Search->isEnabled() == false ) {     // only enable, if disabled, not if dictionary status changed between some and all selected dicts available
    dictSearchClear();                          // re-sets most of the tooltips already
    ui.f_Search->setEnabled( true );
    ui.f_Dict->setEnabled( true );
    ui.l_LookUp->setToolTip( tr( "Enter word to look up. The selected dictionaries (via Settings button below) will be queried while you type." ) );
    ui.lE_LookUp->setToolTip( tr( "" ) );
    ui.lE_LookUp->setFocus();
  }
}

void Qvtwfc::dictSearchDisable()            // disable dict search frame, do not have any configured dicts
{
  dictSearchClear();                            // re-sets most of the tooltips already
  dictResTabHeadClear();
  ui.f_Search->setEnabled( false );
  ui.l_LookUp->setToolTip( tr( "Enable dict lookup and set dictionaries to use in the settings dialog." ) );
  ui.lE_LookUp->setToolTip( tr( "Enable dict lookup and set dictionaries to use in the settings dialog." ) );
}

void Qvtwfc::dictSearchClear()              // clear all widgets in dict search frame, leave table headers
{
  ui.lE_LookUp->clear();
  dictSearchResClear();
  ui.lE_LookUp->setFocus();
}

void Qvtwfc::dictSearchResClear()           // clear dict search result excl. search input widget
{
  dictResTabClear();                            // clear result table in dict search frame, includes ResTrans and set 'Add entry' button
  dictResNewEntryClear();
  dictResAddButSet();
}

void Qvtwfc::dictResTabHeadClear()          // clear result table fully, including header, call dictResTabClear first
{
  ui.tW_DictRes->clear();
  ui.tW_DictRes->setColumnCount( 0 );
  ui.tW_DictRes->setRowCount( 0 );
}

void Qvtwfc::dictResTabClear()              // clear result table, translation and add button in dict search frame
{
  ui.tW_DictRes->clearContents();
  ui.tW_DictRes->setRowCount( 0 );
  ui.tW_DictRes->setToolTip( tr( "" ) );
}

void Qvtwfc::dictResTabSetup()              // setup result table in dict search frame (headers)
{
  ui.tW_DictRes->setColumnCount( 3 );
  slD.clear();                                  // set headers 'Name' and 'Description'
  slD.append( tr( "Translation" ) );
  slD.append( tr( "Orig match" ) );
  slD.append( tr( "Dict" ) );
  ui.tW_DictRes->setHorizontalHeaderLabels( slD );
  ui.tW_DictRes->setColumnWidth( 0, iColRes0Width );
  ui.tW_DictRes->setColumnWidth( 1, iColRes1Width );
  ui.tW_DictRes->setColumnWidth( 2, iColRes2Width );
}

void Qvtwfc::dictResNewEntryClear()         // clear dict new orig lineedit, new word type and new translation at once
{
  ui.lE_ResOrig->clear();

  bTypBlocked = true;                           // if not set already
  ui.cB_WordType->setCurrentIndex( 0 );
  bTypBlocked = false;

  ui.lE_ResTrans->clear();
}

void Qvtwfc::dictResAddButSet()             // enable/disable 'Add Entry' button in dict frame acc. ResOrig and ResTrans content
{
  bAddButSet = false;                           // disable 'Add entry' button
  sD3 =  tr( "Nothing to add" );                // default tooltip

  sD = ui.lE_ResOrig->text();
  sD2 = ui.lE_ResTrans->text();
  if( sD2.isEmpty() == false ) {                // have new translation
    if( sD.isEmpty() == false ) {               // have new original
      if( ui.tW_List->findItems( sD, Qt::MatchFixedString ).count() == 0 or ui.tW_List->findItems( sD, Qt::MatchFixedString ).at( 0 )->column() != 0 ) { // new orig not found at all or at least not in original column
        bAddButSet = true;
        sD3 = tr( "Click to add this wordpair to the list on the left" );
      } else {
        sD3 = tr( "The exact original word/phrase already exists ! Please change translation directly in the table on the left." );
      }
    } else {
      sD3 = tr( "Wordpair incomplete ! No new original word/phrase entry." );
    }
  } else {                                      // no new translation
    if( sD.isEmpty() == false ) {               // have new original
      sD3 = tr( "Wordpair incomplete ! No new translation entry." );
    }
  }

  ui.pB_AddDict->setEnabled( bAddButSet );
  ui.pB_AddDict->setToolTip( sD3 );
}

void Qvtwfc::dictUpdIcon()                  // set dict status LED and tooltip, depending on actual status
{
  switch ( iDS ) {                          // dict status:0=disabled(grey),1=server error(red),2=selection partly avail(orange),3=OK,all selected dbs avail(green)
    case 0:                                     // disabled(grey)
      sD2 = "LED_grey.png";
      ui.l_LED->setToolTip( tr( "dict look up disabled! Enable it in the settings dialog, see 'Settings' button below." ) );
      break;
    case 1:                                     // dict server error (red)
      sD2 = "LED_red.png";
      ui.l_LED->setToolTip( tr( "No or no selected dictionary available! - Look up disabled." ) );
      break;
    case 2:                                     // selection partly available (orange)
      sD2 = "LED_orange.png";
      ui.l_LED->setToolTip( tr( "Warning, not all previously selected dictionaries are available in this session!" ) );
      break;
    case 3:                                     // OK, all selected dbs available (green)
      sD2 = "LED_green.png";
      ui.l_LED->setToolTip( tr( "All selected dictionaries available. dict look up should be OK!" ) );
      break;
  }
  ui.l_LED->setPixmap( QPixmap( "/usr/share/apps/qvtwfc/icons/22x22/actions/" + sD2 ) );
}

void Qvtwfc::dictLookUp()                   // lookup field content changed, start new dict look up
{
  sLookUp = ui.lE_LookUp->text();
//  cout << "qvt: dictLookUp(): reached ! " << sLookUp.toStdString() << endl;      // *****  debug  *****
    if( sLookUp.isEmpty() == false ) {
        dictSearchResClear();                         // clear search result
        ui.lE_ResOrig->setText( sLookUp.trimmed() );  // remove and leading or trailing spaces
        emit sigQvtDicLookUp( sLookUp );
    } else {
      dictSearchClear();
    }
}

void Qvtwfc::dictReceiveMatches( const QList<LookUp>& lookupsD )  // receive word look up result
{
//  cout << "qvt: dictReceiveMatches(): reached ! " << sLookUp.toStdString() << endl;      // *****  debug  *****
  lookups = lookupsD;
  dictTimer5DictStop();                         // received answer from dictd, stop 5 sec timer timer5Dict to unblock further queries if answer from dictd timed out
  iNumMatches = lookups.count();                // number of matches from this result, other dictionaries may follow !
  if( iNumMatches > 0 ) {
    disconnect(ui.tW_DictRes, SIGNAL( itemChanged( QTableWidgetItem* ) ), this, SLOT( dictResTabClicked( QTableWidgetItem* ) ) ); // chk if clicked item is in col 0
    disconnect(ui.tW_DictRes->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timerChkDictResHeadResized() )); // 5s delay t chk col headers w resized->save
    dictResTabClear();
    dictResTabSetup();                          // setup result table in dict search frame (headers)
    ui.tW_DictRes->setRowCount( iNumMatches );
    ui.tW_DictRes->setToolTip( tr( "Check/uncheck the translation result(s) you like to add/remove from the new entry." ) );
    for ( i = 0; i < iNumMatches; ++i) {
      sD = lookups.at( i ).trans;
      sD2 = lookups.at( i ).orig;
      sD3 = lookups.at( i ).dict;
      newItem = new QTableWidgetItem( sD );       // translation
        newItem->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
        newItem->setCheckState( Qt::Unchecked );
        ui.tW_DictRes->setItem( i, 0, newItem);
      newItem2 = new QTableWidgetItem( sD2 );     // original word
        ui.tW_DictRes->setItem( i, 1, newItem2);
      newItem3 = new QTableWidgetItem( sD3 );     // dictionary
        ui.tW_DictRes->setItem( i, 2, newItem3);
    }
    ui.tW_DictRes->resizeRowsToContents();
    connect(ui.tW_DictRes, SIGNAL( itemChanged( QTableWidgetItem* ) ), this, SLOT( dictResTabClicked( QTableWidgetItem* ) ) );   // chk if clicked item is in col 0
    connect(ui.tW_DictRes->horizontalHeader(), SIGNAL( sectionResized( int, int, int ) ), this, SLOT( timerChkDictResHeadResized() )); // 5s timer t chk col head w resized->save
  }
}

void Qvtwfc::dictResOrigUpd()               // update dict lookup resulting new original field
{
}

void Qvtwfc::dictResTabClicked( QTableWidgetItem *item )  // check if clicked item is in col 1 (was 0, but due to crashes 0 now empty...)
{
  qtwiD = new QTableWidgetItem( 0 );
  qtwiD = item;
  if( qtwiD->column() == 0 ) {                  // only react if item in col 0 was clicked
    dictResTransUpd();
  }
}

void Qvtwfc::dictResTransUpd()              // update dict lookup resulting new translation field
{
  iD = ui.tW_DictRes->rowCount();
  if( iD > 0 ) {
    sTrans = "";
    for( i = 0; i < iD; ++i ) {
      if( ui.tW_DictRes->item( i, 0 )->checkState() == Qt::Checked ) {
        if( sTrans != "" ) {                    // not first match, add ' , ' for better readability
          sTrans = sTrans + ", ";
        }
        sTrans = sTrans + ui.tW_DictRes->item( i, 0 )->text();
      }
    }
    ui.lE_ResTrans->setText( sTrans );
    dictResTypUpd();                            // try guess word type if not selected manually
    dictResAddButSet();
  }
}

void Qvtwfc::dictResTypUpd()                // try guess word type if not selected manually, 0=v(erb), 1=n(oun), 2=adj(ective), 3=adv(erb)
{
  if( bTypBlocked == false ) {                  // only try guessing if not already set by user
    iD = sTrans.indexOf( "{" );
    iD2 = sTrans.indexOf( "}" );
    if( iD > -1 and iD2 > -1 ) {                // found at least one {xy} word type definition
      sD = sTrans.mid( iD + 1, iD2 - iD - 1 );
      if( sD == "f" or sD == "m" or sD == "n" ) {
        ui.cB_WordType->setCurrentIndex( 1 );
      } else if( sD == "v" ) {                  // redundant
        ui.cB_WordType->setCurrentIndex( 0 );
      } else if( sD == "adj" ) {
        ui.cB_WordType->setCurrentIndex( 2 );
      } else if( sD == "adv" ) {
        ui.cB_WordType->setCurrentIndex( 3 );
      }
    }
  }
}

void Qvtwfc::dictResTypSet()                // word type for current lookup/results was set manually, do not override it
{
  bTypBlocked = true;                           // word type for current lookup/result(s) was set manually, block override by auto guessing
}

void Qvtwfc::dictResTypIdxChngd()           // word type idx changed set focus to dict lookup, enable F12
{
  ui.lE_LookUp->setFocus();                     // set active widget back to new dict lookup
}

void Qvtwfc::dictTimer5DictStart()          // start 5 sec timer timer5Dict to unblock further queries if answer from dictd timed out
{
  if( timer5Dict->isActive() == false ) {
    timer5Dict->start(3500);
  }
}

void Qvtwfc::dictTimer5DictStop()           // received answer from dictd, stop 5 sec timer timer5Dict to unblock further queries if answer from dictd timed out
{
  if( timer5Dict->isActive() == true ) {
    timer5Dict->stop();
  }
}

void Qvtwfc::dictTimer5DictTimedOut()       // 5 sec timer timer5Dict unblocking next dict query timed out, dict prob no longer avail, but could become avail again
{                                           // need to disable dict lookup here ???

}

void Qvtwfc::dictTimerLookUpClearStart()    // start timerDictLookUpClear, 12 sec delay to clear dict look up input widget under certain conditions
{
  if( timerDictLookUpClear->isActive() == true ) {
    timerDictLookUpClear->stop();
  }
  timerDictLookUpClear->start(18000);
}

void Qvtwfc::dictAutoClearLookUpInput()     // auto clear dict look up input widget after 12sec and certain conditions
{
  if( ui.pB_AddDict->isEnabled() == false and ui.lE_LookUp->hasFocus() == true ) {
    dictSearchClear();
  }
}


// **************************************************************************
// **  local search                                                        **
// **************************************************************************
void Qvtwfc::localSearch( const QString& sD2 )  // search local original, select closes match in orig column to what is typed in the 'Search Original' lineEdit
{
  if( sD2.isEmpty() == false ) {
    ui.lE_SearchTrans->clear();
    if( ui.tW_List->findItems( sD2, Qt::MatchStartsWith ).count() > 0 ) {
      if( ui.tW_List->findItems( sD2, Qt::MatchStartsWith ).at( 0 )->column() == 0 ) {           // only select items in column 0 (originals)
        ui.tW_List->setCurrentItem( ui.tW_List->findItems( sD2, Qt::MatchStartsWith ).at( 0 ) );
        ui.tW_List->scrollToItem( ui.tW_List->currentItem(), QAbstractItemView::PositionAtCenter );
      }
    }
  }
}

void Qvtwfc::localSearchTrans( const QString& sD3 ) // search local translation, select closes match in transl. col to what typed in 'Search Translation' lineEdit
{
  if( sD3.isEmpty() == false ) {
    ui.lE_Search->clear();
    if( ui.tW_List->findItems( sD3, Qt::MatchContains ).count() > 0 ) {
      if( ui.tW_List->findItems( sD3, Qt::MatchContains | Qt::MatchWrap ).at( 0 )->column() == 2 ) {  // only select items in column 1 (translations)
        range = ui.tW_List->findItems( sD3, Qt::MatchContains );
        iTMInd = 0;                                     // translation match index on local translation search
        ui.tW_List->setCurrentItem( range.at( iTMInd ) );
        ui.tW_List->scrollToItem( range.at( iTMInd ), QAbstractItemView::PositionAtCenter );
      }
    }
  }
}

void Qvtwfc::localSearchTransRP()           // search local translation, return pressed, mark next match
{
  if( range.count() > 0 ) {
    iTMInd = iTMInd + 1;
    if( iTMInd > range.count() - 1 ) {
      iTMInd = 0;
    }
    ui.tW_List->setCurrentItem( range.at( iTMInd ) );
    ui.tW_List->scrollToItem( range.at( iTMInd ), QAbstractItemView::PositionAtCenter );
  }
}


// **************************************************************************
// **  wordpair editing mini menus and functions + key press event         **
// **************************************************************************
void Qvtwfc::keyPressEvent( QKeyEvent *ke ) // key press event
{
  switch ( ke->key() ) {
    case Qt::Key_A:
      if( ke->modifiers() ==  Qt::AltModifier ) {
        about();                                      // show about dialog
      }
      break;
    case Qt::Key_D:
      if( ke->modifiers() ==  Qt::AltModifier ) {
        if( ui.tW_List->hasFocus() == true ) {        // ALT + d is pressed while table has focus, ask to delete entry
          wpDel();                                    // ask to delete selected wordpair
        }
      }
      break;
    case Qt::Key_H:
      if( ke->modifiers() ==  Qt::AltModifier ) {
        help();                                       // show help dialog
      }
      break;
    case Qt::Key_Q:
      if( ke->modifiers() ==  Qt::AltModifier ) {
        quitPrep();                                   // quit qvtwfc
      }
      break;
    case Qt::Key_S:
      if( ke->modifiers() ==  Qt::AltModifier ) {
        save();                                       // save word list
      }
      break;
    case Qt::Key_F2:
      if( ui.pB_Save->isEnabled() == true ) {
        save();                                       // save word list
      }
      break;
    case Qt::Key_F10:
      if( ui.f_Search->isEnabled() == true ) {
        dictSearchClear();
      }
      break;
    case Qt::Key_F12:
      if( ui.pB_AddDict->isEnabled() == true ) {
        wpIns();
      }
      break;
  }
}

void Qvtwfc::menuCalled( const QPoint& qpD )  // show menu, right clicked in table widget
{
  qpDM = qpD;
  qtwiD = ui.tW_List->itemAt( qpDM );
  switch ( qtwiD->column() ) {
    case 0:
      menuDelWp->exec( QCursor::pos() );        // delete wordpair menu, column 0
      break;
    case 1:
      menuEditType->exec( QCursor::pos() );     // edit type menu, column 1
      break;
    case 2:
      menuEditTrans->exec( QCursor::pos() );    // edit translation menu, column 2
      break;
  }
}

void Qvtwfc::wpDel()                        // delete currently selected wordpair, ask first!
{
  ranges = ui.tW_List->selectedRanges();
  sD = "";
  iD = ranges.count();
  li_DelWps.clear();
  for( i = 0; i < iD; ++i) {                    // cycle over # of selection ranges
    qtwsrD = ranges.at( i );
    iD2 = qtwsrD.topRow();
    iD3 = qtwsrD.bottomRow();
    for( i2 = iD2; i2 < iD3 + 1; ++i2) {        // cycle over each range: row = index = starting at 0
      sD = sD + ui.tW_List->item( i2, 0 )->text() + " - " +  ui.tW_List->item( i2, 1 )->text() + " - " +  ui.tW_List->item( i2, 2 )->text() + "\n";
      li_DelWps.append( i2 );
    }
  }
  if( li_DelWps.count() > 0 ) {                 // have selection to possible delete
    if(!QMessageBox::question( 0, tr( "Delete wordpair(s) ?" ), 
                                  tr( "\nSure to delete the currently selected wordpair(s) ?\n \n" ) + sD,
                                  tr( "&Yes, Delete" ), tr( "No, &Cancel" ), 0, 1 )) {
      iD = 0;
      for( i = 0; i < li_DelWps.count(); ++i) {
        ui.tW_List->removeRow( li_DelWps.at( i ) - iD );
        iD = iD + 1;
      }
      askSaveTrue();
    }
  } else {
    QMessageBox::information( this, tr( "Nothing to do !" ),tr( "No wordpair to delete !?" ) );
  }
}

void Qvtwfc::wpEdType()                     // show dialog to edit word type of selected wordpair
{
  iD = 0;
  sD = ui.tW_List->itemAt( qpDM )->text();
  if( sD == "v" ) {                             // nothing to do
  } else if( sD == "n" ) {
    iD = 1;
  } else if( sD == "adj" ) {
    iD = 2;
  } else if( sD == "adv" ) {
    iD = 3;
  } else {
    iD = 4;
  }
  uiwt.cB_Type->setCurrentIndex( iD );
  wEdwt.move( qpDM );
  wEdwt.show();
}

void Qvtwfc::wpEdTypeUpd()                  // do update word type of selected wordpair
{
  sD = uiwt.cB_Type->currentText();
  ui.tW_List->itemAt( qpDM )->setText( sD );
  askSaveTrue();
  wpEdTypeCancel();
}

void Qvtwfc::wpEdTypeCancel()               // cancel/hide dialog to edit word type of selected wordpair
{
  wEdwt.hide();
}

void Qvtwfc::wpEdTrans()                    // edit translation of selected wordpair
{
  uiwtrans.lE_Trans->setFocus();
  uiwtrans.lE_Trans->setText( ui.tW_List->itemAt( qpDM )->text() );
  wEdwtrans.move( qpDM );
  wEdwtrans.show();
  uiwtrans.lE_Trans->deselect();
  uiwtrans.lE_Trans->setCursorPosition( 0 );
}

void Qvtwfc::wpEdTransUpd()                 // update translation of selected wordpair
{
  sD = uiwtrans.lE_Trans->text();
  ui.tW_List->itemAt( qpDM )->setText( sD );
  askSaveTrue();
  wpEdTransCancel();
}

void Qvtwfc::wpEdTransCancel()              // cancel edit translation of selected wordpair
{
  wEdwtrans.hide();
}

void Qvtwfc::wpIns()                        // insert manual entered wordpair
{
  sLastIns = ui.lE_ResOrig->text().trimmed();   // orig part of the last insert wordpair - to scroll to
  ui.tW_List->insertRow( 0 );
  QTableWidgetItem *newItem = new QTableWidgetItem( sLastIns ); // new original word
  ui.tW_List->setItem( 0, 0, newItem);
  QTableWidgetItem *newItem2 = new QTableWidgetItem( ui.cB_WordType->currentText().trimmed() ); // new translation
  ui.tW_List->setItem( 0, 1, newItem2);
  QTableWidgetItem *newItem3 = new QTableWidgetItem( ui.lE_ResTrans->text().trimmed() );  // new translation
  ui.tW_List->setItem( 0, 2, newItem3);
  dictSearchClear();
  QTimer::singleShot(200, this, SLOT( wpInsUpd() ) );
}

void Qvtwfc::wpInsUpd()                     // update view to show just inserted new item
{
  ui.lE_LookUp->setFocus();                     // set active widget back to new dict lookup
  localSearch( sLastIns );
}


#include "qvtwfc.moc"
