/****************************************************************************
 **
 **  Copyright (C) 2007-2009 by Helge Kortendieck, alan@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef QVTWFC_H
#define QVTWFC_H

#include <QtAlgorithms>
#include <QAction>
#include <QApplication>
#include <QChar>
#include <QDialog>
#include <QDir>
#include <QEvent>
#include <QFileDialog>
#include <QFileInfo>
#include <QFont>
#include <QHeaderView>
#include <QInputDialog>
#include <QKeyEvent>
#include <QList>
#include <QMenu>
#include <QMessageBox>
#include <QPixmap>
#include <QPoint>
#include <QSettings>
#include <QSizePolicy>
#include <QString>
#include <QStringList>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QTableWidgetSelectionRange>
#include <QTimer>
#include <QWidget>

#include "structs.h"
#include "abouthelp.h"
#include "dict.h"
#include "rw.h"
#include "settings.h"
#include "ui_qvtwfc.h"
#include "ui_askth.h"
#include "ui_edwt.h"
#include "ui_edwtrans.h"

#include <iostream>           // *****   debug   *****
using namespace std;

class Qvtwfc : public QWidget //MainWindow
{
  Q_OBJECT

    public:
      Qvtwfc(QWidget *parent = 0);
//        ~Qvtwfc();

    protected:
      void keyPressEvent( QKeyEvent *ke );



    private:
      Ui::Qvtwfc ui;        // GUI maintenance mode
      Ui::Askth uia;        // dialog to ask table headers / languages
      Ui::Edwt uiwt;        // dialog to edit word type of an existing wordpair
      Ui::Edwtrans uiwtrans; // dialog to edit translation

      bool bAddButSet;      // enable/disable 'Add entry' button
      bool bAskSave;        // changes in tabwidget, ask to save
      bool bDicObj;         // flag whether dict object created
      bool bDictBlock;      // block further dict look up query, while one is still running
      bool bDictUse;        // use dict server
      bool bDlgAh;          // flag whether about/help dialog is shown
      bool bDlgSet;         // flag whether settings dialog is shown
      bool bInit;           // flag whether we are starting up
      bool bInitErr;        // flag whether there was an error during init, if, show message at end
      bool bIns;            // used during sort, have we sorted in this item already?
      bool bNewFile;        // flag whether a new file should be created after saving current
      bool bOpenFile;       // flag whether a file should be opened after saving current
      bool bQuit;           // flag whether we are quitting
      bool bSaveFirst;      // flag whether we need to save before quitting
      bool bTypBlocked;     // word type for current lookup/result(s) was set manually, block override by auto guessing
      bool bWriteErr;       // flag whether there were any write errors

      int i;
      int i2;
      int iAhType;          // about/help dialog type
      int iCol0Width;       // width of original word column
      int iCol1Width;       // width of type column
      int iCol2Width;       // width of translation column
      int iColRes0Width;    // width of nil/translation result column
      int iColRes1Width;    // width of trans/orig match result column
      int iColRes2Width;    // width of orig/dictionary result column
      int iD;               // dummy int
      int iD2;              // dummy int
      int iD3;              // dummy int
      int iD4;              // dummy int
      int iDictErr;         // error code from dict server
      int iDS;              // dict status: 0=disabled, 1=no or none requested dicts avial., 2=selection partly available, 3=OK, all selected dbs available
      int iDSD;             // same as iDSD, but dummy
      int iErr;             // error code
      int iFontSize;        // font size, can be set in settings dialog
      int iInputFocus;      // widget which has last got input focus: 0=undefined, 1=local seach, 2=local seach translation, 3=dict look up
      int iNumMatches;      // number of received matches per dictionary
      int iPM;              // mode, 0 = normal, 1 = test run (check file for test run)
      int iTMInd;           // translation match index on local translation search
      int iTwItemCurRow;    // row of current tablewidgetitem
      int iTwItemCurCol;    // column of current tablewidgetitem, both will become 'PREV' after checks
      int iTwItemPrevRow;   // row of previous tablewidgetitem
      int iTwItemPrevCol;   // column of previous tablewidgetitem
      int iWpCnt;           // count of valid word pairs

      QList<DictDef> dictsD;
      QList<DictDef> dictsD2;

      WordPair wpD;             // struct entity
      QList<WordPair> wplD;     // dummy wordpair list, used for sorting
      QList<WordPair> wordpair;
      QList<LookUp> lookups;    // look up results, corresponding original and translation, will become lookup widget entry
      QList<LookUp> lookupsD;   // look up results, corresponding original and translation, will become lookup widget entry

      QList<QTableWidgetItem *> range;
      QList<QTableWidgetSelectionRange> ranges;
      QList<int> li_DelWps;     // list with table rows to delete

      QAction *actDelWp;    // action for menu to delete selected wordpairs
      QAction *actEditType; // action for menu to edit word type
      QAction *actEditTrans;// action for menu to edit translation

      QDialog wAsk;         // dialog asking for languages
      QDialog wEdwt;        // dialog to edit word type of a wordpair
      QDialog wEdwtrans;    // dialog to edit translation

      QMenu *menuDelWp;     // mini menu to delete item after right mouse click in table
      QMenu *menuEditType;  // mini menu to edit word type after right mouse click in table
      QMenu *menuEditTrans; // mini menu to edit translation after right mouse click in table

      QPoint qpD;           // show delete wordpair menu at this point, used to call origin function
      QPoint qpDM;          // show delete wordpair menu at this point, constant within the program once called via qpD

      QString sD;           // dummy QString
      QString sD1;          // dummy QString
      QString sD2;          // dummy QString
      QString sD3;          // dummy QString
      QString sDictTT;      // tooltip text for dict status LED on main widget
      QString sDir;         // start dir for file dialogs
      QString sErr;         // error message
      QString sErrTit;      // error message titel
      QString sErrTxt;      // error message text
      QString sFile;        // full path and name of voc file
      QString sLastIns;     // orig part of the last insert wordpair - to scroll to
      QString sLookUp;      // word to look up by the dict server
      QString sNf;          // full path and name of new file to open
      QString sO;           // original language
      QString sOf;          // name of file to open, becomes sNf when sure that sNf has not to be saved first
      QString sQuitTit;     // titel of messagebox when quitting and save abort or incomplete
      QString sQuitTxt;     // 1st line in messagebox when quitting and save abort or incomplete
      QString sT;           // translation language
      QString sTrans;       // string with translation for new to be added entry
      QString sTwItemPrevOnEntry; // content of previous tablewidgetitem on entry
      QString sTwItemPrevNow;     // content of previous tablewidgetitem now, after it was left     *****     ever used     ?????     ******
      QString sTwItemCur;         // current twitem (will become sTwItemPrevOnEntry after checks)
      QString sWriteErr;    // write error text

      QStringList slD;            // dummy QStringList

      QTableWidgetItem *newItem;  // dict result trans
      QTableWidgetItem *newItem2; // dict result orig
      QTableWidgetItem *newItem3; // dict result dict
      QTableWidgetItem *qtwiD;    // dummy qtablewidgetitem
      QTableWidgetSelectionRange qtwsrD;

      QTimer *timer5;                   // 5 sec delay to check if col headers were resized -> then save
      QTimer *timer5Dict;               // 5 sec timer to unblock further dict queries when no answer was received, probably lost dict server
      QTimer *timerDictResHeadReSized;  // 3 sec delay to check if col headers in dict result were resized -> then save
      QTimer *timerDictLookUpClear;     // 12sec timer to clear dict look up search input widget, if found solution in local list, avoid to do manual clear
      QTimer *timerSaveWordList;        // 5min timer to save word list, started if word list changed and not already active

      QWidget* qwD;
      QWidget* qwD2;

      aboutHelp *ahw;             // help widget
      dict *dic;                  // dict server handling
      rw *rwf;                    // read/write files
      settings *set;              // settings dialog

    private slots:
      void about();               // show about information
      void help();                // show help message
      void quitDelay();           // delay to allow clean up - needed ????????????????????
      void quitPrep();            // quit clicked clean up
      void quitDo();              // finally quit the program
      void save();                // save clicked

      void ahDlgCreate( int iD ); // about/help dialog create and open, 1=about, 2=help
      void ahDlgClosed();         // about/help dialog closed

      void askLangShow();         // ask original and translation language when creating new voc file
      void askLangSet();          // use original and translation language when creating new voc file
      void askLangCancel();       // ask language dialog canceled

      void askSaveTrue();         // set bAskSave true and enable save button
      void askSaveFalse();        // set bAskSave false and disable save button

      void defActions();          // define all possible actions/menu entries

      void dictCreate();                                            // create dict handling qobject
      void dictConErr( int iDictErr, const QString& sD );           // error connecting to dict server, 1=starting, 2=word lookup, 2=getting list for set dlg
      void dictDictsStatusUpd( int iDSD, const QString& sErr );     // update status of available dictionaries, or if dict server available at all
      void dictQueryErr( int iDSD, const QString& sErr );           // error return codes from dict server during query
      void dictReCon( const QString& sD, int iD );                  // query dict server after dicts button in settings dialog clicked
      void dictReceiveDictsOnServer( const QList<DictDef>& dictsD, const QList<DictDef>& dictsD2 ); // receive all dictionaries avail on dict server, for set dlg
      void dictDisable();                                           // query dict server after dicts button in settings dialog clicked

  // prepare dict look up (enable/disable):
      void dictSearchClear();     // clear all widgets in dict search frame
      void dictSearchDisable();   // disable dict search frame, do not have any configured dicts
      void dictSearchEnable();    // enable dict search frame, have some configured dicts
      void dictSearchResClear();  // clear dict search result excl. search input widget
      void dictResTabClear();     // clear dict result table, spare headers
      void dictResTabClicked( QTableWidgetItem *item ); // check if clicked item is in col 0
      void dictResTabSetup();     // setup result table in dict search frame (headers)
      void dictResTabHeadClear(); // clear dict result table headers
      void dictResNewEntryClear();// clear dict new orig lineedit, new word type and new translation at once
      void dictResAddButSet();    // enable/disable 'Add Entry' button in dict frame acc. ResOrig and ResTrans content

      void dictUpdIcon();         // set widget icon and tooltip for dict status, depending on actual status
  // perform dict look up:
      void dictReceiveMatches( const QList<LookUp>& lookupsD );      // receive look up matches
      void dictLookUp();          // lookup field content changed, start new dict look up
      void dictResOrigUpd();      // update dict lookup resulting new original field
      void dictResTransUpd();     // update dict lookup resulting new translation field
      void dictResTypSet();       // word type for current lookup/results was set manually, do not override it
      void dictResTypIdxChngd();  // word type idx changed set focus to dict lookup, enable F12
      void dictResTypUpd();       // try guess word type if not selected manually, 0=v(erb), 1=n(oun), 2=adj(ective), 3=adv(erb)
      void dictTimer5DictStart(); // start 5 sec timer timer5Dict to unblock further queries if answer from dictd timed out
      void dictTimer5DictStop();  // received answer from dictd, stop 5 sec timer timer5Dict to unblock further queries if answer from dictd timed
      void dictTimer5DictTimedOut();    // 5 sec timer timer5Dict unblocking next dict query timed out, dict prob no longer avail, but could become avail again
      void dictTimerLookUpClearStart(); // start timerDictLookUpClear, 12 sec delay to clear dict look up input widget under certain conditions out
      void dictAutoClearLookUpInput();  // auto clear dict look up input widget after 12sec and certain conditions

      void fileNewClicked();      // icon to create a new vocabulary file was clicked
      void fileOpenClicked();     // icon to open an existing vocabulary file was clicked
      void fileOpenPressed();     // user pressed return in the vocabulary file line edit

      void fileCreateDialog();    // dialog to create new vocabulary file
      void fileGetStartDir();     // get start dir for file new/open dialog
      void fileOpenDialog();      // open the file dialog to load a file
      void fileChkPrep();         // prepare file check

      void localSearch( const QString& sD );      // search local original, select closes match in original column to what is typed in 'Search Original' lineEdit
      void localSearchTrans( const QString& sD ); // search local translation, select closes match in trans col to what is typed in 'Search Translation' lineEdit
      void localSearchTransRP();                  // search local translation, return pressed, mark next match

      void readAbort( const QString& sErrTit, const QString& sErrTxt, int iErr );  // fatal error opening file
      void readWErr( const QString & sErrTit, const QString & sErrTxt, int iErr, const QString& sD1, const QString & sD2 ); // file read with non fatal errors
      void readOk( const QString& sD1, const QString & sD2 );                      // file successfully read, orig lang, trans lang, word list, # word pairs

      void writePrep();           // prepare writing files
      void writeAbort( const QString& sErrTit, const QString& sErrTxt, int iErr ); // fatal error writing file
      void writeWErr( const QString& sErrTit, const QString& sErrTxt, int iErr );  // non fatal error writing file
      void writeOk();             // file successfully written

      void initDelay();           // short timer to allow GUI to come up before proceeding
      void init();                // starting, reading vocabulary file from last session
      void initEnd();             // start sequence ended

      void menuCalled( const QPoint& qpDM );  // right click in table widget, open context menu

      void setDlgOpen();              // open settings dialog
      void setDlgClose();             // close settings dialog
      void setDlgDictErr();           // error while querying the dict server

      void setFontSize();             // set font size of main widget excl. buttons

      void settingsRead();
      void settingsReRead();          // received request from settings dialog to re-read settings
      void settingsWrite();

      void twCellEntered( int iD, int iD2, int iD3, int iD4 );  // check, whether the previous cell was changed and if so, call sort and activate save button
      void twPop();                   // populate text edit with vocabulary file after loading it

      void timer5ChkHeadResized();        // start timer5, 5 sec delay to check if col headers were resized -> then save
      void timerChkDictResHeadResized();  // 5 sec delay timer to check if column header widths of dict result widget have been changed
      void chkHeadResized();              // timer5 timed out, check if col headers were resized -> then save
      void chkDictResHeadResized();       // timer5 timed out, check if col headers were resized -> then save

      void widgetChgd( QWidget* qwD, QWidget* qwD2 ); // new widget got focus, upd input after save if req.

      void wpDel();           // delete currently selected wordpair, ask first!
      void wpEdType();        // edit word type of selected wordpair
      void wpEdTypeUpd();     // do update word type of selected wordpair
      void wpEdTypeCancel();  // cancel/hide dialog to edit word type of selected wordpair
      void wpEdTrans();       // edit translation of selected wordpair
      void wpEdTransUpd();    // update translation of selected wordpair
      void wpEdTransCancel(); // cancel/hide dialog to edit translation of selected wordpair
      void wpIns();           // insert manual entered wordpair
      void wpInsUpd();        // update view to show just inserted new item

    signals:
      void sigQvtDicReCon( const QString& sD, int iD ); // re-connect to dict server with new settings
      void sigQvtDicLookUp( const QString& sLookUp );   // look up word
      void sigQvtDicDisCon( int iD );                   // disconnect - disabling dict lookup/closing
      void sigQvtDicReReadSet();                        // let dic re-read settings too, triggered by settings dlg

      void sigQvtSendDictsToSetDlg( const QList<DictDef>& dictsD, const QList<DictDef>& dictsD2 );  // send avail dicts from server to set dialog, no eval before

      void sigAhDlg( int iAhType );   // show dialog 1=about, 2=help
      void sigAhDlgClose();           // close about/help dialog

      void sigDlgErr( int iFontSize );// show dialog with long error text   ****************  should go away  *******************
};

#endif 
