/****************************************************************************
 **
 **  Copyright (C) 2007 by Helge Kortendieck, qvtwfc@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef QVTWFCT_H
#define QVTWFCT_H

#include <cstdlib>

#include <QDir>
#include <QFileDialog>
#include <QFileInfo>
#include <QMessageBox>
#include <QSettings>
#include <QString>      // not needed?
#include <QStringList>  // not needed?
#include <QTime>        // needed for random generator
#include <QTimer>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QTextEdit>
#include <QWidget>

#include "structs.h"
#include "ui_qvtwfct.h"
#include "rw.h"

using namespace std;

class Qvtwfct : public QWidget //MainWindow
{
  Q_OBJECT

    public:
      Qvtwfct(QWidget *parent = 0);
//        ~Qvtwfct();
      QString sNf;          // full path and name of new file to open

    private slots:
      void quitDo();        // quit the program
      void quitPrep();      // clean up before quit

      void errorNOut();     // show error message and quit

      void fileChkPrep();   // prepare file check
      void readAbort( const QString& sErrTit, const QString& sErrTxt, int iErr );
                                // fatal error opening file
      void readWErr( const QString & sErrTit, const QString & sErrTxt, int iErr, const QString& sD1, const QString & sD2 ); // file read with non fatal errors
      void readOk( const QString& sD1, const QString & sD2 );
                                // file successfully read, orig lang, trans lang, word list, # word pairs

      void initDelay();     // short timer to allow GUI to come up before proceeding
      void init();          // starting, reading vocabulary file from last session

      void settingsRead();
      void settingsWrite();

      void trainPrep();     // prepare trainings session

      void fcClear();       // clear widget on start up
      void fcLoopBegin();   // begin loop (test run)
      void fcGetPair();     // get word pair: original and translation
      void fcShowOrig();    // show original lang word
      void fcShowTrans();   // show translation
      void fcLoopNext();    // next clicked, show translation, next new word or exit

    private:
      Ui::Qvtwfct ui;       // GUI flash card mode

      bool bOk;             // used for argument conversion

      int iArgs;            // number of command line args, 1st always program name
      int iCnt;             // test loop counter - number of words asked
      int iD;               // dummy int
      int iD2;              // dummy int
      int iIterT;           // # iterations (words to ask) in test mode before exit from settings
      int iIterA;           // # iterations (words to ask) in test mode before exit from command line arguments
      int iIterU;           // # iterations (words to ask) in test mode before exit to be used
      int iLine;            // line in qstringlist from which word pair to be pickt
      int iNextStep;        // which step to go next after clicking next, step 1 orig or step 2 transl.
      int iWpCnt;           // number of word pairs

      uint uiD;             // dummy uint
      uint uiErr;           // return error code

      QList<WordPair> wordpair;

      QString sD;           // dummy QString
      QString sD1;          // dummy QString
      QString sD2;          // dummy QString
      QString sD3;          // dummy QString
      QString sErr;         // error message
      QString sErrTit;      // error message titel
      QString sErrTxt;      // error message text
      QString sFile;        // full path and name of voc file used in normal mode
      QString sFileA;       // full path and name of voc file from command line arguments
      QString sFileT;       // full path and name of voc file for test run
      QString sFileU;       // full path and name of voc file to use in this test run
      QString sI;           // number of iterations as string
      QString sO;           // original word/phrase
      QString sT;           // translation

      QPoint ptPos;         // widget position
      QSize seSize;         // widget size

      rw *rwf;              // read/write files
};

#endif 
